/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#include <glib.h>
#include <string.h>
#include "namespace.h"
#include "numexp-client1.h"

extern NumexpClient *client;

GList* nxp_client_completion_complete(char const *text, char** prefix)
{
    GList *current_completions = NULL;
    GCompletion *comp;
    char *namespace, *ns;
    char *text2complete, *t2c;
    int size = strlen(NXP_IDENTIFIER_PATH_UP) + strlen(NXP_IDENTIFIER_PATH_SEP);
    int skipped = 0;

    namespace = strdup(client->cwd);
    ns = namespace;

    text2complete = strdup(text);
    t2c = text2complete;

    if (strstr(text2complete, NXP_IDENTIFIER_PATH_UP)) {
	*prefix = strdup(text2complete);
	while (strncmp(text2complete,
		       NXP_IDENTIFIER_PATH_UP NXP_IDENTIFIER_PATH_SEP, size) == 0) {
	    char *last = namespace;
	    char *current;

	    skipped += size;
	    text2complete += size;

	    if (strcmp(last, NXP_IDENTIFIER_PATH_SEP) != 0) {
		*(last + strlen(last) - strlen(NXP_IDENTIFIER_PATH_SEP)) = '\0';
		while ((current = strstr(last, NXP_IDENTIFIER_PATH_SEP)))
		    last = current + strlen(NXP_IDENTIFIER_PATH_SEP);
		*last = '\0';
	    } else {
		g_free(ns);
		g_free(t2c);
		return NULL;
	    }
	}
	*(*prefix + skipped) = '\0';
    }

    if (strstr(text2complete, NXP_IDENTIFIER_PATH_SEP)) {
	char* last = text2complete;
	char* current;
	while ((current = strstr(last, NXP_IDENTIFIER_PATH_SEP)))
	    last = current + strlen(NXP_IDENTIFIER_PATH_SEP);

	namespace = strdup(text2complete);
	namespace[last - text2complete] = '\0';
	numexp_client_get_completions(client, namespace);

	*prefix = namespace;
	text2complete = (text2complete + strlen(namespace) );

    }

      /* Get the completions for the current working directory */
    comp = g_hash_table_lookup(client->completions, namespace);
    if (!comp) {
	numexp_client_get_completions(client, namespace);
	comp = g_hash_table_lookup(client->completions, namespace);
    }

      /* Calculate completions */
    current_completions = g_completion_complete(comp, text2complete, NULL);

      /* There are pointer to the beginning of the string! */
    g_free(ns);
    g_free(t2c);

    return current_completions;
}


void nxp_client_completion_init(NumexpClient* self) {
    self->completions = g_hash_table_new(g_str_hash, g_str_equal);
}

    /* This clear the namespace if it exists */
void nxp_client_completion_new_namespace(NumexpClient* self, const char *ns) {
    GCompletion* comp;
    if ((comp = g_hash_table_lookup(self->completions, ns))) {
	  /* it is already there... empty them */
	g_completion_clear_items(comp);

    } else {
	g_hash_table_insert(self->completions, 
			    strdup(ns),
			    comp = g_completion_new(NULL));
    }
}

void nxp_client_completion_add_items(NumexpClient* self, const char *ns, GList* l) {
    GCompletion *comp;
    comp = g_hash_table_lookup(self->completions, ns);
    if (!comp) {
	nxp_client_completion_new_namespace(self, ns);
	comp = g_hash_table_lookup(self->completions, ns);
    }
    if (l) {
	  /* g_completion_add_items doesn't like empty lists */
	g_completion_add_items(comp, l);
    }
}

