/* -*- Mode: C; c-file-style: "stroustrup"; -*- */
#define _GNU_SOURCE
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif
#include <string.h>

#include "numexp-client1.h"

#define PARENT_TYPE BONOBO_OBJECT_TYPE

static GObjectClass *parent_class;

static void numexp_client_class_init(NumexpClientClass *class);

extern void changePrompt(void);


static void numexp_client_finalize(GObject *object)
{
/*      NumexpClient *self = NUMEXP_CLIENT(object); */

    if (parent_class->finalize)
	parent_class->finalize(object);
}


static void numexp_client_init(NumexpClient *self)
{
    nxp_client_completion_init(self);
    /* was empty */
}


GType numexp_client_get_type(void)
{
    static GType type = 0;

    if (!type) {
	GTypeInfo info = {
	    sizeof(NumexpClientClass), 
	    (GBaseInitFunc) NULL,
	    (GBaseFinalizeFunc) NULL, 
	    (GClassInitFunc) numexp_client_class_init, 
	    NULL, /* class_finalize */
	    NULL, /* class_data */
	    sizeof(NumexpClient), 
	    0, /* n_preallocs */
	    (GInstanceInitFunc) numexp_client_init,
	    NULL
	};

	type = bonobo_type_unique(
	    PARENT_TYPE, 
	    POA_Numexp_Client__init, NULL, 
	    G_STRUCT_OFFSET(NumexpClientClass, epv), 
	    &info, "NumexpClient");
    }

    return type;
}


BonoboObject* numexp_client_new(void)
{
    BonoboObject *obj;
    NumexpClient *self;

    obj = g_object_new(NUMEXP_CLIENT_TYPE, NULL);
    self = NUMEXP_CLIENT(obj);

    return obj;
}


static void impl_Numexp_Client_print(PortableServer_Servant     servant,
				     const CORBA_char          *str,
				     Numexp_Client_MessageType  msg_type,
				     CORBA_Environment         *ev)
{
    switch (msg_type)
    {
    case Numexp_Client_MESSAGE_PRINT:
	printf("%s\n", str);
	break;
    case Numexp_Client_MESSAGE_PARSER_WARNING:
	printf("Parser warning: %s\n", str);
	break;
    case Numexp_Client_MESSAGE_EVAL_WARNING:
	printf("Warning: %s\n", str);
	break;
    }
}


static Numexp_Element impl_Numexp_Client_evalFunction(PortableServer_Servant servant,
						      const Numexp_DefinitionHandle def,
						      const Numexp_Client_ArgList *args,
						      CORBA_Environment * ev)
{
    Numexp_Element retval;
    Numexp_Element *elem;
    NumexpClient *self = NUMEXP_CLIENT(bonobo_object(servant));
    CORBA_Environment ev1;
    int i;

    g_message("%s (definition %llu): %u argument(s)", __FUNCTION__, def, args->_length);
    for (i = 0; i < args->_length; ++i) {
	elem = args->_buffer + i;
	printf("Arg %i: %i\n", i, (int) (*elem));
    }
    CORBA_exception_init(&ev1);
    retval = Numexp_Kernel_elementString(self->kernel, "(test CORBA client function result)", &ev1);
    CORBA_exception_free(&ev1);
    return retval;
}



void numexp_client_get_completions(NumexpClient *self, const char *ns)
{
    Numexp_DefinitionList *defs;
    Numexp_Definition *def;
    GList *l;
    CORBA_Environment ev1;
    int i;

    /* Check if the namespace already exists, and if it, clear it */
    nxp_client_completion_new_namespace(self, ns);

    /* Get namespace listing */
    CORBA_exception_init(&ev1);
    defs  = Numexp_Kernel_namespaceListing(self->kernel, ns, &ev1);
    if (ev1._major != CORBA_NO_EXCEPTION) {
	CORBA_exception_free(&ev1);
	return;
    }

    CORBA_exception_free(&ev1);
    /* create new completion list */
    for (l = NULL, i = 0; i < defs->_length; ++i) {
	GString* str;
	def = defs->_buffer + i;
	str = g_string_new(def->name);
	if (def->type == Numexp_DEFINITION_NAMESPACE) {
	    str = g_string_append(str,"::");
	} else if (def->type == Numexp_DEFINITION_BUILT_IN_FUNCTION ||
		   def->type == Numexp_DEFINITION_USER_FUNCTION) {
	    str = g_string_append_c(str,'[');
	} else {
	    str = g_string_append_c(str,' ');
	}
	l = g_list_prepend(l, str->str);
	g_string_free(str, FALSE);
    }
    nxp_client_completion_add_items(self,ns,l);
    CORBA_free(defs);
}


static void
impl_Numexp_Client_currentNamespaceNotify(PortableServer_Servant servant,
					  const Numexp_Namespace * ns,
					  CORBA_Environment * ev)
{
    NumexpClient *self = NUMEXP_CLIENT(bonobo_object(servant));
    self->cwd = strdup(ns->name);

    changePrompt();

}

static void
impl_Numexp_Client_namespaceNotify(PortableServer_Servant servant,
				   const Numexp_Namespace * ns,
				   CORBA_Environment * ev)
{
    NumexpClient *self = NUMEXP_CLIENT(bonobo_object(servant));

    /* Completions contain functions only in the root namespace */
    if (strcmp(ns->name, "::") != 0) return;
    /* Update completion list */
    numexp_client_get_completions(self, ns->name);
}



static void numexp_client_class_init(NumexpClientClass *class)
{
    GObjectClass *object_class = (GObjectClass *) class;
    POA_Numexp_Client__epv *epv = &class->epv;

    parent_class = g_type_class_peek_parent(class);

    object_class->finalize = numexp_client_finalize;

    epv->print		        = impl_Numexp_Client_print;
    epv->evalFunction 	        = impl_Numexp_Client_evalFunction;
    epv->namespaceNotify        = impl_Numexp_Client_namespaceNotify;
    epv->currentNamespaceNotify = impl_Numexp_Client_currentNamespaceNotify;

}



