#include "derive.h"
#include "types/int.h"
#include "types/tree.h"
#include "types/function.h"

NxpOpDeriveFunc derivation_table[NXP_NUM_OPS];

NxpElement* elem_LN (NxpElement *el) {
    NxpElement *args = nxp_element_list_new();
    NxpElement *retval;
    nxp_element_list_append(args, el);
    retval = nxp_element_function_new(nxp_identifier_new_simple("ln"), args, NULL);
    NXP_ELEMENT_UNREF(args);
    return retval;
}

NxpOpDeriveFunc get_derive_function(NxpOp op) {
    return derivation_table[op - 1];
}

NxpElement* nxp_derive_sum(NxpOp op,
			   NxpIdentifier *id,
			   NxpElement *left,
			   NxpElement *right,
			   gpointer user_data,
			   GError **error)
{
    /* TODO -- error handling in all these functions */
    return elem_PLUS(nxp_element_derive(left ,id,user_data,NULL),
		     nxp_element_derive(right,id,user_data,NULL));
}

NxpElement* nxp_derive_sub(NxpOp op,
			   NxpIdentifier *id,
			   NxpElement *left,
			   NxpElement *right,
			   gpointer user_data,
			   GError **error)
{
    return elem_MINUS(nxp_element_derive(left ,id,user_data,NULL),
		      nxp_element_derive(right,id,user_data,NULL));
}

NxpElement* nxp_derive_uminus(NxpOp op,
			      NxpIdentifier *id,
			      NxpElement *left,
			      NxpElement *right,
			      gpointer user_data,
			      GError **error)
{
    g_return_val_if_fail(right,NULL);
    return elem_UMINUS(nxp_element_derive(right,id,user_data,NULL));
}

NxpElement* nxp_derive_star(NxpOp op,
			    NxpIdentifier *id,
			    NxpElement *left,
			    NxpElement *right,
			    gpointer user_data,
			    GError **error)
{
    return elem_PLUS(elem_TIMES(nxp_element_derive(left,id,user_data,NULL),NXP_ELEMENT_REF(right)),
		     elem_TIMES(NXP_ELEMENT_REF(left),nxp_element_derive(right,id,user_data,NULL)));
    
}

NxpElement* nxp_derive_slash(NxpOp op,
			     NxpIdentifier *id,
			     NxpElement *left,
			     NxpElement *right,
			     gpointer user_data,
			     GError **error)
{
    return elem_OVER(elem_MINUS(elem_TIMES(NXP_ELEMENT_REF(right),
					   nxp_element_derive(left,id,user_data,NULL)),
				elem_TIMES(NXP_ELEMENT_REF(left),
					   nxp_element_derive(right,id,user_data,NULL))),
		     elem_CIRC(NXP_ELEMENT_REF(right),
			       nxp_element_int_new(2)));
}

NxpElement *nxp_derive_hat(NxpOp op,
			   NxpIdentifier *id,
			   NxpElement *left,
			   NxpElement *right,
			   gpointer user_data,
			   GError **error)
{
    NxpElement *el = elem_TIMES(elem_CIRC(NXP_ELEMENT_REF(left),
					  elem_PLUS(nxp_element_int_new(-1),
						    NXP_ELEMENT_REF(right))),
				elem_PLUS(elem_TIMES(NXP_ELEMENT_REF(right),
						     nxp_element_derive(left, id, user_data, NULL)),
					  elem_TIMES(NXP_ELEMENT_REF(left),
						     elem_TIMES(elem_LN(NXP_ELEMENT_REF(left)),
								nxp_element_derive(right, id, user_data, NULL)))));
    return el;
}


void nxp_op_derivation_init(void) {
    derivation_table[NXP_OP_PLUS       - 1] = &nxp_derive_sum;
    derivation_table[NXP_OP_MINUS      - 1] = &nxp_derive_sub;
    derivation_table[NXP_OP_MINUS_LEFT - 1] = &nxp_derive_uminus;
    derivation_table[NXP_OP_STAR       - 1] = &nxp_derive_star;
    derivation_table[NXP_OP_SLASH      - 1] = &nxp_derive_slash;
    derivation_table[NXP_OP_HAT        - 1] = &nxp_derive_hat;
    derivation_table[NXP_OP_DOT3       - 1] = NULL;
    derivation_table[NXP_OP_PLUS2_LEFT - 1] = NULL;
    derivation_table[NXP_OP_MINUS2_LEFT- 1] = NULL;
    derivation_table[NXP_OP_PLUS2_RIGHT- 1] = NULL;
    derivation_table[NXP_OP_MINUS2_RIGHT-1] = NULL;
    derivation_table[NXP_OP_GT         - 1] = NULL;
    derivation_table[NXP_OP_GT_EQ      - 1] = NULL;
    derivation_table[NXP_OP_LT_EQ      - 1] = NULL;
    derivation_table[NXP_OP_LT_EQ      - 1] = NULL;
    derivation_table[NXP_OP_EQ2        - 1] = NULL;
    derivation_table[NXP_OP_EXCL_EQ    - 1] = NULL;
    derivation_table[NXP_OP_SLASH_BSLASH-1] = NULL;
    derivation_table[NXP_OP_BSLASH_SLASH-1] = NULL;
    derivation_table[NXP_OP_TILDE_LEFT - 1] = NULL;
    derivation_table[NXP_OP_EXCL_RIGHT - 1] = NULL;
    derivation_table[NXP_OP_EQ         - 1] = NULL;
    derivation_table[NXP_OP_COLON_EQ   - 1] = NULL;
    derivation_table[NXP_OP_PERCENT    - 1] = NULL;

}
