/* -*- Mode: C; c-file-style: "bsd"; -*- */

#include "typecasts.h"
#include <glib.h>


static GHashTable *casters = NULL;

#define MAKE_CASTER_KEY(type_left, type_right) \
	GUINT_TO_POINTER( (((guint32)(type_left)) << 16) | ((guint32)type_right) )


void nxp_typecasts_init(void)
{
    if (!casters)
	casters = g_hash_table_new(NULL, NULL);
}

/* This function is just needed by nxp_typecasts_heritate_caster */
static void copy_typecasts(gpointer key, gpointer value, gpointer data)
{
    gpointer new_key;

    if ((GPOINTER_TO_UINT(key) & 0xffff0000) == (GPOINTER_TO_UINT(data) & 0xffff0000)) {
	new_key = MAKE_CASTER_KEY((GPOINTER_TO_UINT(data) & 0xffff),
				  (GPOINTER_TO_UINT(key) & 0xffff));
	g_hash_table_insert(casters, new_key, value);
    }
}

/* Propagates typecasts on new objects, for example, we import gmp-r, so we
 * have new real numbers and numexp must know how to convert them to complex
 * numbers */
void nxp_typecasts_heritate_caster(NxpElementType parent, NxpElementType child)
{
    g_hash_table_foreach(casters, copy_typecasts, MAKE_CASTER_KEY(parent, child));
}

void nxp_typecasts_register_caster(NxpElementType from, NxpElementType to, 
				   NxpCasterFunc caster)
{
    GSList *children, *lst;

    g_hash_table_insert(casters, MAKE_CASTER_KEY(from, to), caster);
    children = nxp_element_type_get_children_types(from);
    for (lst = children; lst; lst = lst->next)
	nxp_typecasts_register_caster(GPOINTER_TO_INT(lst->data), to, caster);
    g_slist_free(children);
}

NxpCasterFunc inline nxp_typecasts_get_caster(NxpElementType from, NxpElementType to)
{
    return g_hash_table_lookup(casters,	MAKE_CASTER_KEY(from, to));
}


NxpElement* nxp_typecasts_try_cast(NxpElement *from, NxpElementType to_type)
{
    NxpCasterFunc caster;

    g_return_val_if_fail(from, NULL);
    caster = nxp_typecasts_get_caster(NXP_ELEMENT_GET_TYPE(from), to_type);

    return caster? caster(from) : NULL;
}

gboolean nxp_typecasts_exists(NxpElementType from, NxpElementType to)
{
    NxpCasterFunc caster;
    caster = nxp_typecasts_get_caster(from, to);
    return caster?TRUE:FALSE;
}
