/* -*- Mode: C; c-file-style: "stroustrup"; -*- */

#include <stdio.h>
#include "number.h"
#include "tree.h"
#include "int.h"
#include "message.h"
#include "op_bindings.h"
#include "types_misc.h"
#include "errors.h"
#include "parser/plugins.h"
#include "i18n.h"


static NxpElementTypeInfo *typeinfo = NULL;

static NxpElement* nxp_number_derive(NxpElement *a, NxpIdentifier *var,
				     NxpEvalContext *context, GError **error)
{
    return nxp_element_int_new(0);
}

int nxp_element_number_compare_int(NxpElementNumber *self, nxp_int_t nb)
{
    g_return_val_if_fail(NXP_ELEMENT_NUMBER_TYPE_INFO(self)->compare_int, 0);
    return NXP_ELEMENT_NUMBER_TYPE_INFO(self)->compare_int(self, nb);
}

gboolean nxp_element_default_number_is_integer(NxpElementNumber *self)
{
    return FALSE;
}

static NxpElement *substitute_func(NxpElement *elem, NxpElement *var,
				   NxpElement *el, NxpEvalContext *context,
				   GError **error)
{
    return NXP_ELEMENT_REF(elem);
}

static gboolean dependson_func(NxpElement *self, NxpElement *var,
			       NxpEvalContext *context, GError **error)
{
    return FALSE;
}

gboolean nxp_element_number_is_integer(NxpElementNumber *self)
{
    g_return_val_if_fail(NXP_ELEMENT_NUMBER_TYPE_INFO(self)->is_integer, FALSE);
    return NXP_ELEMENT_NUMBER_TYPE_INFO(self)->is_integer(self);
}

NxpElement* nxp_element_number_opposed(NxpElementNumber *self)
{
    g_return_val_if_fail(NXP_ELEMENT_NUMBER_TYPE_INFO(self)->opposed, NULL);
    return NXP_ELEMENT_NUMBER_TYPE_INFO(self)->opposed(self);
}

int nxp_element_number_get_type(void)
{
    static int nxp_element_type_number = 0;

    if (nxp_element_type_number == 0) {
	nxp_element_type_number = nxp_element_next_type();
	typeinfo = nxp_element_type_register_info(
		nxp_element_type_number, /* type to declare */
		sizeof(NxpElementNumberTypeInfo), /* size of our typeinfo */
		0,	/* types from that it comes : here, it's nothing */
		sizeof(NxpElementTypeInfo)); /* size of the parent typeinfo */
    }
    return nxp_element_type_number;
}

void nxp_element_number_type_init()
{
    NxpElementNumberTypeInfo *nb_typeinfo;
    typeinfo = nxp_element_type_get_info(nxp_element_number_get_type());
    nb_typeinfo = (NxpElementNumberTypeInfo *)typeinfo;
    typeinfo->name = g_strdup("number");
    typeinfo->eval_func = NULL;
    typeinfo->simplify_func = NULL;
    typeinfo->substitute_func = substitute_func;
    typeinfo->derive_func = nxp_number_derive;
    typeinfo->str_func = NULL;
    typeinfo->repr_func = NULL;
    typeinfo->truth_func = NULL;
    typeinfo->dependson_func = dependson_func;
    nb_typeinfo->compare_int = NULL;
    nb_typeinfo->is_integer = nxp_element_default_number_is_integer;
    nb_typeinfo->opposed = NULL;
}
