/* -*- c -*- */
#include <gsl/gsl_qrng.h>
#include "nxp_kernel.h"
#include "gsl.h"


static NxpElement *
nxp_gsl_qrng_name(ETC)
{
    gsl_qrng       *qrng = nxp_eval_context_get_user_data(context, "qRNG");
    if (qrng) {
	NxpElement     *el = nxp_element_string_new(gsl_qrng_name(qrng));
	GSL_RETURN(el);
    } else {
	return nxp_element_string_new("-none-");
    }
}

static NxpElement *
nxp_gsl_qrng_size(ETC)
{
    gsl_qrng       *qrng = nxp_eval_context_get_user_data(context, "qRNG");
    if (qrng) {
	unsigned int   *usize =
	    nxp_eval_context_get_user_data(context, "qRNG-size");
	NxpElement     *el = nxp_element_int_new(*usize);
	GSL_RETURN(el);
    } else {
	return nxp_element_int_new(0);
    }
}

static NxpElement *
nxp_gsl_qrng_init_get(ETC)
{
    double         *buffer;
    gsl_qrng       *qrng = nxp_eval_context_get_user_data(context, "qRNG");
    if (qrng) {
	unsigned int   *usize =
	    nxp_eval_context_get_user_data(context, "qRNG-size");
	NxpElement     *vec;

	buffer = g_new(double, *usize);
	gsl_qrng_get(qrng, buffer);
	vec = nxp_double_buffer_to_vector(buffer, *usize);
	GSL_RETURN(vec);
    } else {
	return NULL;
    }
}

static NxpElement *
nxp_gsl_qrng_init_niederreiter_2(NxpElementInt * dimension, ETC)
{
    unsigned int   *usize = g_new(unsigned int, 1);
    gsl_qrng       *qrng = nxp_eval_context_get_user_data(context, "qRNG");
    if (qrng) {
	/*
	 * Free it 
	 */
	gsl_qrng_free(qrng);
	g_free(nxp_eval_context_get_user_data(context, "qRNG-size"));
	qrng = NULL;
    }
    *usize = (unsigned int) ABS(nxp_element_r_get_value_as_int(dimension));
    qrng = gsl_qrng_alloc(gsl_qrng_niederreiter_2, *usize);
    nxp_eval_context_set_user_data(context, "qRNG", qrng);
    nxp_eval_context_set_user_data(context, "qRNG-size", usize);
    GSL_RETURN(nxp_element_void_new());
}

// #PROTOTYPE <VOID> qrng_reset() WITH NAME reset
// static NxpElement* nxp_gsl_qrng_reset(ETC) {
// gsl_qrng *qrng = nxp_eval_context_get_user_data(context, "qRNG");
// // if (qrng) gsl_qrng_init(qrng);
// GSL_RETURN(nxp_element_void_new());
// }

static NxpElement *
nxp_gsl_qrng_init_sobol(NxpElementInt * dimension, ETC)
{
    unsigned int   *usize = g_new(unsigned int, 1);
    gsl_qrng       *qrng = nxp_eval_context_get_user_data(context, "qRNG");
    if (qrng) {
	/*
	 * Free it 
	 */
	gsl_qrng_free(qrng);
	g_free(nxp_eval_context_get_user_data(context, "qRNG-size"));
	qrng = NULL;
    }
    *usize = (unsigned int) ABS(nxp_element_r_get_value_as_int(dimension));
    qrng = gsl_qrng_alloc(gsl_qrng_sobol, *usize);
    nxp_eval_context_set_user_data(context, "qRNG", qrng);
    nxp_eval_context_set_user_data(context, "qRNG-size", usize);
    GSL_RETURN(nxp_element_void_new());
}

void
nxp_gsl_qrng_init(NxpEvalContext * context)
{
    gsl_qrng       *qrng;
    NxpIdentifier  *id;
    unsigned int   *size;

    /*
     * it starts empty!! This means that I don't know when to free it :-/ 
     */
    qrng = NULL;
    size = g_new(unsigned int, 1);
    *size = 0;
    nxp_eval_context_set_user_data(context, "qRNG", qrng);
    nxp_eval_context_set_user_data(context, "qRNG-size", size);

nxp_name_space_new(context->cwd,"gsl");
nxp_name_space_new(nxp_name_space_find_child(context->cwd,"gsl"),"qRNG");
id = nxp_identifier_new_with_full_path(3,"gsl","qRNG", "current");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_qrng_name, NULL, nxp_gsl_qrng_name, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","qRNG", "currentSize");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_qrng_size, NULL, nxp_gsl_qrng_size, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","qRNG", "sample");
 nxp_bifunc_define(id, &nxp_prototype__VOID, nxp_gsl_qrng_init_get, NULL, nxp_gsl_qrng_init_get, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","qRNG", "init_niederreiter2");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_qrng_init_niederreiter_2, NULL, nxp_gsl_qrng_init_niederreiter_2, context, NULL); nxp_identifier_unref(id);
id = nxp_identifier_new_with_full_path(3,"gsl","qRNG", "init_sobol");
 nxp_bifunc_define(id, &nxp_prototype__INT, nxp_gsl_qrng_init_sobol, NULL, nxp_gsl_qrng_init_sobol, context, NULL); nxp_identifier_unref(id);
}
