// -*- Mode: c; c-file-style: "stroustrup" -*-
#include <stdlib.h>
#include <stdio.h>
#include "files.h"
#include "parser.h"
#include "errors.h"

NxpElement* nxp_files_rc_load(NxpEvalContext *context, GError **error)
{
    char *filename;
    NxpElement* ret;
    GError *erro = NULL;

    filename = g_strdup_printf("%s/.numexprc", getenv("HOME"));
    ret = nxp_files_load(filename, context, &erro);
    g_free(filename);
    if (erro) {
	g_propagate_error(error, erro);
	NXP_ELEMENT_XUNREF(ret);
	return NULL;
    }
    return ret;
}


NxpElement* nxp_files_load(const char *file, NxpEvalContext *context, GError **error)
{
    gchar      *contents;
    NxpElement *parsed, *result;
    GError *err1 = NULL;

    g_return_val_if_fail(file, NULL);
    if (!g_file_get_contents(file, &contents, NULL, &err1)) {
	g_set_error(error, nxp_error_domain, NXP_ERROR_FILE_NOT_FOUND,
		    "%s", g_strdup(err1->message));
	g_error_free(err1);
	return NULL;
    }
    parsed = exp_parse_string(contents, context, &err1);
    g_free(contents);
    if (err1) {
	g_propagate_error(error, err1);
	NXP_ELEMENT_XUNREF(parsed);
	return NULL;
    }
    result = nxp_element_eval(parsed, context, &err1);
    NXP_ELEMENT_UNREF(parsed);
    if (err1) {
	g_propagate_error(error, err1);
	NXP_ELEMENT_XUNREF(result);
	return NULL;
    }
    return result;
}
